@@ -42,12 +42,16 @@ class soupManager:
         - It provides various methods to extract data and discover elements within the source code.
     """
 
-    def __init__(self, url=None,url_mgr=None, source_code=None, req_mgr=None, parse_type="html.parser",soup=None,soup_mgr=None):
-        self.url = url
-        self.url_mgr = get_url_mgr(url=self.url)
-        self.req_mgr = req_mgr or requestManager(url_mgr=self.url_mgr,url=self.url,source_code=source_code)
+class soupManager:
+    def __init__(self, url=None, url_mgr=None, source_code=None, req_mgr=None, parse_type="html.parser", soup=None, soup_mgr=None):
+        self.url_mgr = get_url_mgr(url=url, url_mgr=url_mgr)
+        self.url = self.url_mgr.url
+        self.req_mgr = req_mgr or requestManager(url_mgr=self.url_mgr, url=self.url, source_code=source_code)
         self.source_code = (source_code or (req_mgr.source_code if req_mgr else "")) or ""
+        self.parse_type = parse_type
         self.soup = BeautifulSoup(self.source_code, parse_type)
+        self._all_links_data = None
+        self._meta_tags_data = None
 
     def all_meta(self):
         out = []
@@ -67,23 +71,50 @@ class soupManager:
                 out.setdefault(k, []).append(m["content"])
         return out
 
+
+
+    @property
     def all_links(self):
-        res = []
+        if self._all_links_data is None:
+            self._all_links_data = self._all_links_get()
+        return self._all_links_data
+
+    def _all_links_get(self):
+        links = []
         for l in self.soup.find_all("link"):
             rel = l.get("rel")
             if isinstance(rel, list): rel = " ".join(rel)
-            res.append({
+            links.append({
                 "rel": rel, "href": l.get("href"),
                 "type": l.get("type"), "title": l.get("title"), "hreflang": l.get("hreflang")
             })
-        return res
+        return links
+
+    def get_all_website_links(self, tag="a", attr="href") -> list:
+        """Return all same-domain URLs found in anchors (or given tag/attr)."""
+        urls = []
+        domain = self.url_mgr.domain
+        for t in self.find_all(tag):
+            href = t.get(attr)
+            if not href:
+                continue
+            href = self.url_mgr.get_relative_href(self.url_mgr.url, href)
+            if not self.url_mgr.is_valid_url(href):
+                continue
+            if domain and domain not in href:
+                continue
+            if href not in urls:
+                urls.append(href)
+        return urls
 
     def all_jsonld(self):
         blocks = []
         for s in self.soup.find_all("script", type=re.compile("application/ld\\+json", re.I)):
             txt = s.get_text(strip=True)
-            try: blocks.append(json.loads(txt))
-            except Exception: blocks.append({"raw": txt})
+            try:
+                blocks.append(json.loads(txt))
+            except Exception:
+                blocks.append({"raw": txt})
         return blocks
     def re_initialize(self):
         self.soup= BeautifulSoup(self.source_code, self.parse_type)
@@ -115,61 +146,46 @@ class soupManager:
     def update_parse_type(self,parse_type):
         self.parse_type=parse_type
         self.re_initialize()
-    @property
-    def all_links(self):
-        """This is a property that provides access to the _all_links_data attribute.
-        The first time it's accessed, it will load the data."""
-        if self._all_links_data is None:
-            print("Loading all links for the first time...")
-            self._all_links_data = self._all_links_get()
-        return self._all_links_data
-    def _all_links_get(self):
-        """A method that loads the data (can be replaced with whatever data loading logic you have)."""
-        return self.get_all_website_links()
-    def get_all_website_links(self,tag="a",attr="href") -> list:
-        """
-        Returns all URLs that are found on the specified URL and belong to the same website.
 
-        Args:
-            url (str): The URL to search for links.
 
-        Returns:
-            list: A list of URLs that belong to the same website as the specified URL.
-        """
-        all_urls=[self.url_mgr.url]
+
+
+    def get_all_website_links(self, tag="a", attr="href") -> list:
+        """Return all same-domain URLs found in anchors (or given tag/attr)."""
+        urls = []
         domain = self.url_mgr.domain
-        all_desired=self.get_all_website_links(tag=tag,attr=attr)
-        for tag in all_desired:
-            href = tag.attrs.get(attr)
-            if href == "" or href is None:
-                # href empty tag
+        
+        for t in self.find_all(tag):
+            href = t.get(attr)
+            if not href:
                 continue
-            href=self.url_mgr.get_relative_href(self.url_mgr.url,href)
+            href = self.url_mgr.get_relative_href(self.url_mgr.url, href)
             if not self.url_mgr.is_valid_url(href):
-                # not a valid URL
                 continue
-            if href in all_urls:
-                # already in the set
+            if domain and domain not in href:
                 continue
-            if domain not in href:
-                # external link
-                continue
-            all_urls.append(href)
-                
-        return all_urls
+            if href not in urls:
+                urls.append(href)
+            
+        return urls
+
+
 
 
     @property
     def meta_tags(self):
-        """This is a property that provides access to the _all_links_data attribute.
-        The first time it's accessed, it will load the data."""
         if self._meta_tags_data is None:
-            print("Loading all links for the first time...")
-            self._meta_tags_data = self._all_links_get()
+            self._meta_tags_data = self._meta_tags_get()
         return self._meta_tags_data
+
     def _meta_tags_get(self):
-        """A method that loads the data (can be replaced with whatever data loading logic you have)."""
-        return self.get_meta_tags()
+        meta = {}
+        for tag in self.soup.find_all("meta"):
+            for attr, val in tag.attrs.items():
+                meta.setdefault(attr, [])
+                if val not in meta[attr]:
+                    meta[attr].append(val)
+        return meta
     def get_meta_tags(self):
         tags = self.find_all("meta")
         for meta_tag in tags:
@@ -180,34 +196,35 @@ class soupManager:
                     self.meta_tags[attr].append(values)
 
                     
-    def find_all(self,element,soup=None):
-        soup = self.soup if soup == None else soup
+
+
+    def find_all(self, element, soup=None):
+        soup = self.soup if soup is None else soup
         return soup.find_all(element)
-    def get_class(self,class_name,soup=None):
-        soup = self.soup if soup == None else soup
-        return soup.get(class_name)
+
+    def get_class(self, class_name, soup=None):
+        soup = self.soup if soup is None else soup
+        # Return the "class" attribute (list) of a Tag if present
+        return soup.get(class_name) if hasattr(soup, "get") else None
     @staticmethod
     def has_attributes(tag, *attrs):
         return any(tag.has_attr(attr) for attr in attrs)
+
     def get_find_all_with_attributes(self, *attrs):
         return self.soup.find_all(lambda t: self.has_attributes(t, *attrs))
-    def find_tags_by_attributes(self, tag: str = None, attr: str = None, attr_values: List[str] = None) ->List:
-        if not tag:
-            tags = self.soup.find_all(True)  # get all tags
-        else:
-            tags = self.soup.find_all(tag)  # get specific tags
-            
-        extracted_tags = []
+
+    def find_tags_by_attributes(self, tag: str = None, attr: str = None, attr_values: list[str] | None = None) -> list:
+        tags = self.soup.find_all(tag) if tag else self.soup.find_all(True)
+        out = []
         for t in tags:
             if attr:
-                attribute_value = t.get(attr)
-                if not attribute_value:  # skip tags without the desired attribute
+                val = t.get(attr)
+                if not val:
                     continue
-                if attr_values and not any(value in attribute_value for value in attr_values):  # skip tags without any of the desired attribute values
+                if attr_values and not any(v in val for v in (val if isinstance(val, list) else [val]) for v in attr_values):
                     continue
-            extracted_tags.append(t)
-        return extracted_tags
-
+            out.append(t)
+        return out
 
     def extract_elements(self,url:str=None, tag:str=None, class_name:str=None, class_value:str=None) -> list:
         """
